@@ -210,10 +210,10 @@ def get_existing_keywords(googleads_client, customer_id):
 
 def get_all_audience(client, customer_id):
     data = {
-        "audience_id":list(),
-        "audience_name":list(),
-        "audience_resource_name":list(),
-        "audience_status":list(),
+        "audience_id": list(),
+        "audience_name": list(),
+        "audience_resource_name": list(),
+        "audience_status": list(),
     }
     ga_service = client.get_service("GoogleAdsService")
     query = """
@@ -237,9 +237,10 @@ def get_all_audience(client, customer_id):
         data['audience_resource_name'].append(row.audience.resource_name)
         data['audience_status'].append(row.audience.status.name)
 
-    data= pd.DataFrame(data)
+    data = pd.DataFrame(data)
     return data
 
+
 def get_campaign_name(googleads_client, customer_id, campaign_id):
     ga_service = googleads_client.get_service("GoogleAdsService")
     query = f"""
@@ -257,7 +258,7 @@ def get_campaign_name(googleads_client, customer_id, campaign_id):
     for batch in stream:
         for row in batch.results:
             return str(row.campaign.name)
-        
+
 
 def get_campaign_budget(googleads_client, customer_id, campaign_id):
     ga_service = googleads_client.get_service("GoogleAdsService")
@@ -276,9 +277,9 @@ def get_campaign_budget(googleads_client, customer_id, campaign_id):
     for batch in stream:
         for row in batch.results:
             return float(row.campaign_budget.amount_micros) / 1000000
-        
 
-def create_keyword(googleads_client,customer_id,ad_group_id, keyword_text, kw_type,negative=False):
+
+def create_keyword(googleads_client, customer_id, ad_group_id, keyword_text, kw_type, negative=False):
     ad_group_service = googleads_client.get_service("AdGroupService")
     ad_group_criterion_service = googleads_client.get_service(
         "AdGroupCriterionService")
@@ -1276,6 +1277,175 @@ class Audience():
         return member
 
 
+class BatchOperations:
+
+    def __init__(self,
+        googleads_client,
+        customer_id,
+    ) -> None:
+        self.googleads_client = googleads_client
+        self.customer_id = customer_id
+        self.budget_temporary_id = None
+        self.campaign_temporary_id = None
+        self.ad_group_temporary_id = None
+        self.keyword_temorary_id = None
+        self.latest_temporary_id = -1
+        self.mutate_operations = list()
+
+    def run(self):
+        googleads_service = self.googleads_client.get_service("GoogleAdsService")
+
+        if len(self.mutate_operations) > 0:
+            response = googleads_service.mutate(
+                customer_id=self.customer_id,
+                mutate_operations=self.mutate_operations
+            )
+
+            self._print_response_details(response)
+        else:
+            print("No operations to run")
+
+    def _get_latest_temp_id(self):
+        id = str(self.latest_temporary_id)
+        self.latest_temporary_id -= 1
+        return id
+
+    def create_budget(self, budget_name, budget_dollars):
+        campaign_budget_service = self.googleads_client.get_service(
+            "CampaignBudgetService")
+
+        mutate_operation = self.googleads_client.get_type("MutateOperation")
+        campaign_budget_operation = mutate_operation.campaign_budget_operation
+        campaign_budget = campaign_budget_operation.create
+        campaign_budget.name = budget_name
+        campaign_budget.delivery_method = (
+            self.googleads_client.enums.BudgetDeliveryMethodEnum.STANDARD
+        )
+        campaign_budget.amount_micros = int(budget_dollars * 1000000)
+        campaign_budget.explicitly_shared = False
+        self.budget_temporary_id = self._get_latest_temp_id()
+        campaign_budget.resource_name = campaign_budget_service.campaign_budget_path(
+            self.customer_id, self.budget_temporary_id)
+        
+        self.mutate_operations.append(mutate_operation)
+
+    def create_campaign(self, campaign_name, budget_id=None):
+
+        # Create campaign.
+        campaign_service = self.googleads_client.get_service("CampaignService")
+        mutate_operation = self.googleads_client.get_type("MutateOperation")
+        campaign = mutate_operation.campaign_operation.create
+        campaign.name = campaign_name
+        campaign.advertising_channel_type = (
+            self.googleads_client.enums.AdvertisingChannelTypeEnum.SEARCH
+        )
+        self.campaign_temporary_id = self._get_latest_temp_id()
+        campaign.resource_name = campaign_service.campaign_path(
+            self.customer_id, self.campaign_temporary_id
+        )
+
+        # Recommendation: Set the campaign to PAUSED when creating it to prevent
+        # the ads from immediately serving. Set to ENABLED once you've added
+        # targeting and the ads are ready to serve.
+        campaign.status = self.googleads_client.enums.CampaignStatusEnum.PAUSED
+
+        # Set the bidding strategy and budget.
+        campaign.manual_cpc.enhanced_cpc_enabled = True
+        budget_id = budget_id if budget_id else self.budget_temporary_id
+        campaign.campaign_budget = campaign_service.campaign_budget_path(
+            self.customer_id, budget_id
+        )
+
+        # Set the campaign network options.
+        campaign.network_settings.target_google_search = True
+        campaign.network_settings.target_search_network = True
+        campaign.network_settings.target_content_network = False
+        campaign.network_settings.target_partner_search_network = False
+
+        # Optional: Set the start date.
+        start_time = datetime.date.today() + datetime.timedelta(days=1)
+        campaign.start_date = datetime.date.strftime(start_time, _DATE_FORMAT)
+
+        # Optional: Set the end date.
+        end_time = start_time + datetime.timedelta(weeks=4)
+        campaign.end_date = datetime.date.strftime(end_time, _DATE_FORMAT)
+
+        self.mutate_operations.append(mutate_operation)
+
+    def create_adgroup(self, ad_group_name, cpc_bid=10000000,campaign_id=None):
+        ad_group_service = self.googleads_client.get_service("AdGroupService")
+        campaign_service = self.googleads_client.get_service("CampaignService")
+
+        # Create ad group.
+        mutate_operation = self.googleads_client.get_type("MutateOperation")
+        ad_group = mutate_operation.ad_group_operation.create
+        ad_group.name = ad_group_name
+        ad_group.status = self.googleads_client.enums.AdGroupStatusEnum.ENABLED
+        self.ad_group_temporary_id = self._get_latest_temp_id()
+        ad_group.resource_name = ad_group_service.ad_group_path(
+            self.customer_id, self.ad_group_temporary_id
+        )
+        campaign_id = campaign_id if campaign_id else self.campaign_temporary_id
+        ad_group.campaign = campaign_service.campaign_path(
+            self.customer_id, campaign_id)
+        ad_group.type_ = self.googleads_client.enums.AdGroupTypeEnum.SEARCH_STANDARD
+        ad_group.cpc_bid_micros = cpc_bid
+
+        self.mutate_operations.append(mutate_operation)
+
+    def create_keyword(self, keyword_text, kw_type, ad_group_id=None):
+        ad_group_service = self.googleads_client.get_service("AdGroupService")
+        ad_group_criterion_service = self.googleads_client.get_service(
+            "AdGroupCriterionService")
+
+        # Create keyword.
+        mutate_operation = self.googleads_client.get_type("MutateOperation")
+        ad_group_criterion = mutate_operation.ad_group_criterion_operation.create
+        # ad_group_criterion.resource_name = ad_group_criterion_service.ad_group_criterion_path(
+        #     customer_id,self.keyword_temorary_id
+        # )
+        ad_group_id = ad_group_id if ad_group_id else self.ad_group_temporary_id
+        ad_group_criterion.ad_group = ad_group_service.ad_group_path(
+            self.customer_id, ad_group_id
+        )
+        self.keyword_temorary_id = self._get_latest_temp_id()
+
+        ad_group_criterion.status = self.googleads_client.enums.AdGroupCriterionStatusEnum.ENABLED
+        ad_group_criterion.keyword.text = keyword_text
+        if kw_type == 'PHRASE':
+            kw_type = self.googleads_client.enums.KeywordMatchTypeEnum.PHRASE
+        elif kw_type == 'BROAD':
+            kw_type = self.googleads_client.enums.KeywordMatchTypeEnum.BROAD
+        else:
+            kw_type = self.googleads_client.enums.KeywordMatchTypeEnum.EXACT
+        ad_group_criterion.keyword.match_type = (
+            kw_type
+        )
+        # ad_group_criterion.negative = negative
+        self.mutate_operations.append(mutate_operation)
+
+    def _print_response_details(self, response):
+        """Prints the details of a MutateGoogleAdsResponse.
+        Parses the "response" oneof field name and uses it to extract the new
+        entity's name and resource name.
+        Args:
+            response: a MutateGoogleAdsResponse object.
+        """
+        # Parse the Mutate response to print details about the entities that
+        # were created by the request.
+        suffix = "_result"
+        for result in response.mutate_operation_responses:
+            for field_descriptor, value in result._pb.ListFields():
+                if field_descriptor.name.endswith(suffix):
+                    name = field_descriptor.name[: -len(suffix)]
+                else:
+                    name = field_descriptor.name
+                print(
+                    f"Created a(n) {convert_snake_case_to_upper_case(name)} with "
+                    f"{str(value).strip()}."
+                )
+
+
 def add_negative_keywords(client, customer_id, shared_set_id, keywords, kw_type):
     shared_set_service = client.get_service("SharedSetService")
     shared_criterion_service = client.get_service("SharedCriterionService")
@@ -1754,7 +1924,8 @@ def remove_ad(client, customer_id, ad_group_id, criterion_id):
 
     print(f"Removed ad {ad_group_ad_response.results[0].resource_name}.")
 
-def get_search_term_report(client, customer_id,start_date='2022-01-01', end_date='2022-01-10', full=False):
+
+def get_search_term_report(client, customer_id, start_date='2022-01-01', end_date='2022-01-10', full=False):
     df_dict = {
         'date': list(),
         'stv_resource_name': list(),
@@ -1818,7 +1989,7 @@ def get_search_term_report(client, customer_id,start_date='2022-01-01', end_date
         FROM search_term_view
         WHERE campaign.status IN ('ENABLED','PAUSED') AND
         """
-        if full: 
+        if full:
             query2 = f" segments.date >= '{str(start)[0:10]}' AND segments.date < '{str(end)[0:10]}'"
         else:
             query2 = f" segments.date >= '{str(start)[0:10]}' AND segments.date < '{str(start+step)[0:10]}'"
