@@ -1,7 +1,9 @@
 import os
 import re
+import resource
 import time
 import nltk
+import uuid
 import argparse
 import numpy as np
 import pandas as pd
@@ -17,14 +19,15 @@ import sys
 from google.ads.googleads.client import GoogleAdsClient
 from google.ads.googleads.errors import GoogleAdsException
 
-def get_keywords_from_api_and_url(email, 
-        api_key, 
-        seed_keywords, 
-        depth, 
-        scrape, 
-        urls, 
-        exclude):
-    
+
+def get_keywords_from_api_and_url(email,
+                                  api_key,
+                                  seed_keywords,
+                                  depth,
+                                  scrape,
+                                  urls,
+                                  exclude):
+
     client = RestClient(email, api_key)
 
     def get_keywords(keyword, depth=depth, location='United States'):
@@ -49,18 +52,18 @@ def get_keywords_from_api_and_url(email,
         response3 = client.post(
             "/v3/dataforseo_labs/keyword_suggestions/live", post_data)
 
-        return {  
+        return {
             'related': response,
             'ideas': response2,
             'suggestions': response3
         }
-        
+
     def extract_keywords(responses):
         key_list = []
         sources = []
 
         # print(responses)
-        
+
         if responses['related']["status_code"] == 20000 and responses['related']['tasks'][0]['result'][0]['items']:
             for x in range(len(responses['related']['tasks'][0]['result'][0]['items'])):
                 res = responses['related']['tasks'][0]['result'][0]['items'][x]['related_keywords']
@@ -68,19 +71,21 @@ def get_keywords_from_api_and_url(email,
                     key_list.extend(res)
             print(f"{len(key_list)} related")
             sources.extend(['related' for _ in range(len(key_list))])
-            
+
         if responses['ideas']["status_code"] == 20000 and responses['ideas']['tasks'][0]['result'][0]['items']:
             # not good ideas
-            print(f"{len(responses['ideas']['tasks'][0]['result'][0]['items'])} ideas")
+            print(
+                f"{len(responses['ideas']['tasks'][0]['result'][0]['items'])} ideas")
             for x in range(len(responses['ideas']['tasks'][0]['result'][0]['items'])):
                 res = responses['ideas']['tasks'][0]['result'][0]['items'][x]['keyword']
                 if res is not None:
                     key_list.append(res)
             sources.extend(['ideas' for _ in range(
                 len(responses['ideas']['tasks'][0]['result'][0]['items']))])
-                    
+
         if responses['suggestions']["status_code"] == 20000 and responses['suggestions']['tasks'][0]['result'][0]['items']:
-            print(f"{len(responses['suggestions']['tasks'][0]['result'][0]['items'])} suggestions")
+            print(
+                f"{len(responses['suggestions']['tasks'][0]['result'][0]['items'])} suggestions")
             for x in range(len(responses['suggestions']['tasks'][0]['result'][0]['items'])):
                 res = responses['suggestions']['tasks'][0]['result'][0]['items'][x]['keyword']
                 if res is not None:
@@ -91,30 +96,30 @@ def get_keywords_from_api_and_url(email,
         #     print("error. Code: %d Message: %s" %
         #           (responses['all]["status_code"], responses['all']["status_message"]))
         temp = {
-            'Keywords' : key_list,
-            'Sources' : sources
+            'Keywords': key_list,
+            'Sources': sources
         }
         df = pd.DataFrame.from_dict(temp)
         return df
-    
-    def add_spaces(text, thresh = 3, clean_n = False):
+
+    def add_spaces(text, thresh=3, clean_n=False):
         cleaned = ''
         temp = [l.isupper() for l in text]
         chk = 0
         for i, s in enumerate(temp):
             if s and i != 0 and (i - chk) > thresh:
-                cleaned += ' ' + ext[chk : i]
+                cleaned += ' ' + ext[chk: i]
                 chk = i
         for i, w in enumerate(cleaned):
             if w != ' ':
                 cleaned = cleaned[i:]
                 break
             else:
-                i+=1
+                i += 1
         if clean_n:
             cleaned = cleaned.replace('\n', ' ')
         return cleaned.replace('  ', ' ')
-    
+
     def clean(text):
         text = text.replace('*', '')
         text = text.replace('\ufeff', '')
@@ -136,8 +141,8 @@ def get_keywords_from_api_and_url(email,
         text = text.replace('$', '')
         text = text.replace(',', '')
         return text
-    
-    def _extract_(urls, depth = 1, return_urls = True, return_redirects = True, exclude = exclude):
+
+    def _extract_(urls, depth=1, return_urls=True, return_redirects=True, exclude=exclude):
         print(urls)
         exclude = exclude
         scrape_urls = urls
@@ -154,28 +159,29 @@ def get_keywords_from_api_and_url(email,
                         if exc in url:
                             # print(url)
                             chk = 1
-                    
+
                     if chk == 0:
                         html = urlopen(url).read()
                         # print(html)
                         soup = BeautifulSoup(html, features="html.parser")
-                        
-                        for link in soup.find_all('a', attrs = {'href':re.compile('^/')}):
+
+                        for link in soup.find_all('a', attrs={'href': re.compile('^/')}):
                             uri = link.get('href')
                             temp_urls.append(url + uri)
                             # print(uri)
-                        
+
                         for link in soup.find_all('a', attrs={'href': re.compile('^https://')}):
                             uri = link.get('href')
                             temp_urls.append(uri)
-                        
+
                         for script in soup(['script', 'style']):
                             script.extract()
-                        
+
                         text = soup.get_text()
-                        
+
                         lines = (line.strip() for line in text.splitlines())
-                        chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
+                        chunks = (phrase.strip()
+                                  for line in lines for phrase in line.split("  "))
                         text = '\n'.join(chunk for chunk in chunks if chunk)
                         full_text += ' ' + text
                         resp_urls.append(url)
@@ -184,9 +190,9 @@ def get_keywords_from_api_and_url(email,
                     resp_urls.append(url)
                     resp.append(0)
                     continue
-        
+
             scrape_urls = list(set(temp_urls))
-            
+
         if return_urls:
             if return_redirects:
                 return full_text, resp_urls, (resp_urls, resp)
@@ -196,8 +202,8 @@ def get_keywords_from_api_and_url(email,
             return full_text, (resp_urls, resp)
         else:
             return full_text
-    
-    os.makedirs('data',exist_ok=True)
+
+    os.makedirs('data', exist_ok=True)
     keyword_list = seed_keywords
     df_api = pd.DataFrame(columns=['Keywords', 'Keywords2', 'Sources'])
 
@@ -211,7 +217,7 @@ def get_keywords_from_api_and_url(email,
             df_api = pd.concat([df_api, extracted])
         except:
             print(f"error in {keyword}")
-    
+
     fin_ngrams = []
     if scrape:
         new_urls = []
@@ -226,10 +232,10 @@ def get_keywords_from_api_and_url(email,
         ext, _, _ = _extract_(urls, depth=depth)
         # print(ext)
         ext = add_spaces(ext, clean_n=True)
-        text_path = os.path.join(os.getcwd(),'data/text.txt')
+        text_path = os.path.join(os.getcwd(), 'data/text.txt')
         with open(text_path, 'w', encoding="utf-8") as f:
             f.write(ext)
-            
+
         nltk.download('stopwords')
         stop = set(stopwords.words('english'))
         text = clean(ext)
@@ -238,18 +244,18 @@ def get_keywords_from_api_and_url(email,
         for t in splt_text:
             if t not in stop and not t == '' and not t.isdigit() and len(t) > 1:
                 nw_list.append(t)
-                
+
         ngrams = []
         ngrams.extend(nltk.ngrams(nw_list, 3))
         ngrams.extend(nltk.ngrams(nw_list, 4))
-        
+
         # print(fin_ngrams)
         for ngram in ngrams:
             fin_ngrams.append(' '.join([ng for ng in ngram]))
-        
+
     df_scrape = pd.DataFrame(columns=['Keywords'], data=fin_ngrams)
 
-    return df_api,df_scrape
+    return df_api, df_scrape
 
 
 # Location IDs are listed here:
@@ -330,12 +336,13 @@ def get_keyword_ideas(
         #     "competition."
         # )
         all_keywords.append(idea.text)
-        all_avg_monthly_searches.append(idea.keyword_idea_metrics.avg_monthly_searches)
+        all_avg_monthly_searches.append(
+            idea.keyword_idea_metrics.avg_monthly_searches)
         all_competitions_values.append(competition_value)
 
-    df = pd.DataFrame({"Keywords":all_keywords,"Keywords2":keyword_texts[0],
-                    "volume":all_avg_monthly_searches,
-                    "competition_value":all_competitions_values})
+    df = pd.DataFrame({"Keywords": all_keywords, "Keywords2": keyword_texts[0],
+                       "volume": all_avg_monthly_searches,
+                       "competition_value": all_competitions_values})
     return df
 
 
@@ -363,10 +370,11 @@ def _map_locations_ids_to_resource_names(client, location_ids):
     ).geo_target_constant_path
     return [build_resource_name(location_id) for location_id in location_ids]
 
+
 def get_keywords_from_google(googleads_client,
-                            customer_id,seed_keywords,
-                            lanugage_id=_DEFAULT_LANGUAGE_ID,
-                            location_id=_DEFAULT_LOCATION_IDS):
+                             customer_id, seed_keywords,
+                             lanugage_id=_DEFAULT_LANGUAGE_ID,
+                             location_id=_DEFAULT_LOCATION_IDS):
 
     # GoogleAdsClient will read the google-ads.yaml configuration file in the
     # home directory if none is specified.
@@ -376,13 +384,13 @@ def get_keywords_from_google(googleads_client,
         print(f"Collecting keyword ideas for {keyword[0]}")
         try:
             df = get_keyword_ideas(
-                    googleads_client,
-                    customer_id=customer_id,
-                    location_ids=location_id,
-                    language_id=lanugage_id,
-                    keyword_texts=keyword,
-                    page_url=None
-                )
+                googleads_client,
+                customer_id=customer_id,
+                location_ids=location_id,
+                language_id=lanugage_id,
+                keyword_texts=keyword,
+                page_url=None
+            )
             df_google = df_google.append(df)
             time.sleep(2)
         except GoogleAdsException as ex:
@@ -399,9 +407,10 @@ def get_keywords_from_google(googleads_client,
 
     return df_google
 
-def get_keywords_metrics(email,api_key,df,match_extract):
 
-    client = RestClient(email,api_key)
+def get_keywords_metrics(email, api_key, df, match_extract):
+
+    client = RestClient(email, api_key)
 
     keyword_metrics = {
         'Keywords': list(),
@@ -411,7 +420,7 @@ def get_keywords_metrics(email,api_key,df,match_extract):
         'high_bid': list()
     }
     cpc_metrics = {
-        'Keywords' : list()
+        'Keywords': list()
     }
     values = ['ctr', 'cpc', 'impressions', 'cost', 'clicks']
     for match in match_extract:
@@ -422,7 +431,7 @@ def get_keywords_metrics(email,api_key,df,match_extract):
                 }
             )
 
-    def get_cpc(keywords_list, match = 'exact', bid = 999.0):
+    def get_cpc(keywords_list, match='exact', bid=999.0):
         post_data = dict()
         post_data[len(post_data)] = dict(
             location_name="United States",
@@ -436,8 +445,9 @@ def get_keywords_metrics(email,api_key,df,match_extract):
         if response["status_code"] == 20000:
             return (response)
         else:
-            print("error. Code: %d Message: %s" % (response["status_code"], response["status_message"]))
-        
+            print("error. Code: %d Message: %s" %
+                  (response["status_code"], response["status_message"]))
+
     def get_volume(keywords_list):
         post_data = dict()
         post_data[len(post_data)] = dict(
@@ -453,8 +463,7 @@ def get_keywords_metrics(email,api_key,df,match_extract):
             return(response)
         else:
             print("error. Code: %d Message: %s" %
-                (response["status_code"], response["status_message"]))
-
+                  (response["status_code"], response["status_message"]))
 
     def extract_volume(response):
         results = response['tasks'][0]['result']
@@ -486,17 +495,18 @@ def get_keywords_metrics(email,api_key,df,match_extract):
             extract_volume(volume)
             for c in match_extract:
                 cpc = get_cpc(keywords_list, match=c)
-                extract_cpc(cpc)   
+                extract_cpc(cpc)
     else:
         response = get_volume(keywords_list)
         extract_volume(response)
         for c in match_extract:
             cpc = get_cpc(keywords_list, match=c)
             extract_cpc(cpc)
-    
+
     fin_df_1 = pd.DataFrame(keyword_metrics)
     fin_df_2 = pd.DataFrame(cpc_metrics)
-    fin_df = pd.merge(left=fin_df_1, right=fin_df_2, how='left', on=['Keywords'])
+    fin_df = pd.merge(left=fin_df_1, right=fin_df_2,
+                      how='left', on=['Keywords'])
 
     df = pd.merge(left=df, right=fin_df, how='left', on=['Keywords'])
 
@@ -608,6 +618,236 @@ def get_keywords_metrics2(email, api_key, df, match_extract):
 
     return df
 
+
+def get_keywords_metrics_google(client, customer_id, keywords):
+
+    def _create_keyword_plan(client, customer_id):
+        """Adds a keyword plan to the given customer account.
+
+        Args:
+            client: An initialized instance of GoogleAdsClient
+            customer_id: A str of the customer_id to use in requests.
+
+        Returns:
+            A str of the resource_name for the newly created keyword plan.
+
+        Raises:
+            GoogleAdsException: If an error is returned from the API.
+        """
+        keyword_plan_service = client.get_service("KeywordPlanService")
+        operation = client.get_type("KeywordPlanOperation")
+        keyword_plan = operation.create
+
+        keyword_plan.name = f"Keyword plan for traffic estimate {uuid.uuid4()}"
+
+        forecast_interval = (
+            client.enums.KeywordPlanForecastIntervalEnum.NEXT_QUARTER
+        )
+        keyword_plan.forecast_period.date_interval = forecast_interval
+
+        response = keyword_plan_service.mutate_keyword_plans(
+            customer_id=customer_id, operations=[operation]
+        )
+        resource_name = response.results[0].resource_name
+
+        print(f"Created keyword plan with resource name: {resource_name}")
+
+        return resource_name
+
+    def _create_keyword_plan_campaign(client, customer_id, keyword_plan):
+        """Adds a keyword plan campaign to the given keyword plan.
+
+        Args:
+            client: An initialized instance of GoogleAdsClient
+            customer_id: A str of the customer_id to use in requests.
+            keyword_plan: A str of the keyword plan resource_name this keyword plan
+                campaign should be attributed to.create_keyword_plan.
+
+        Returns:
+            A str of the resource_name for the newly created keyword plan campaign.
+
+        Raises:
+            GoogleAdsException: If an error is returned from the API.
+        """
+        keyword_plan_campaign_service = client.get_service(
+            "KeywordPlanCampaignService"
+        )
+        operation = client.get_type("KeywordPlanCampaignOperation")
+        keyword_plan_campaign = operation.create
+
+        keyword_plan_campaign.name = f"Keyword plan campaign {uuid.uuid4()}"
+        keyword_plan_campaign.cpc_bid_micros = 1000000
+        keyword_plan_campaign.keyword_plan = keyword_plan
+
+        network = client.enums.KeywordPlanNetworkEnum.GOOGLE_SEARCH
+        keyword_plan_campaign.keyword_plan_network = network
+
+        geo_target = client.get_type("KeywordPlanGeoTarget")
+        # Constant for U.S. Other geo target constants can be referenced here:
+        # https://developers.google.com/google-ads/api/reference/data/geotargets
+        geo_target.geo_target_constant = "geoTargetConstants/2840"
+        keyword_plan_campaign.geo_targets.append(geo_target)
+
+        # Constant for English
+        language = "languageConstants/1000"
+        keyword_plan_campaign.language_constants.append(language)
+
+        response = keyword_plan_campaign_service.mutate_keyword_plan_campaigns(
+            customer_id=customer_id, operations=[operation]
+        )
+
+        resource_name = response.results[0].resource_name
+
+        print(
+            f"Created keyword plan campaign with resource name: {resource_name}")
+
+        return resource_name
+
+    def _create_keyword_plan_ad_group(client, customer_id, keyword_plan_campaign):
+        """Adds a keyword plan ad group to the given keyword plan campaign.
+
+        Args:
+            client: An initialized instance of GoogleAdsClient
+            customer_id: A str of the customer_id to use in requests.
+            keyword_plan_campaign: A str of the keyword plan campaign resource_name
+                this keyword plan ad group should be attributed to.
+
+        Returns:
+            A str of the resource_name for the newly created keyword plan ad group.
+
+        Raises:
+            GoogleAdsException: If an error is returned from the API.
+        """
+        operation = client.get_type("KeywordPlanAdGroupOperation")
+        keyword_plan_ad_group = operation.create
+
+        keyword_plan_ad_group.name = f"Keyword plan ad group {uuid.uuid4()}"
+        keyword_plan_ad_group.cpc_bid_micros = 2500000
+        keyword_plan_ad_group.keyword_plan_campaign = keyword_plan_campaign
+
+        keyword_plan_ad_group_service = client.get_service(
+            "KeywordPlanAdGroupService"
+        )
+        response = keyword_plan_ad_group_service.mutate_keyword_plan_ad_groups(
+            customer_id=customer_id, operations=[operation]
+        )
+
+        resource_name = response.results[0].resource_name
+
+        print(
+            f"Created keyword plan ad group with resource name: {resource_name}")
+
+        return resource_name
+
+    def _create_keyword_plan_ad_group_keywords(client, customer_id, plan_ad_group,keywords):
+        """Adds keyword plan ad group keywords to the given keyword plan ad group.
+
+        Args:
+            client: An initialized instance of GoogleAdsClient
+            customer_id: A str of the customer_id to use in requests.
+            plan_ad_group: A str of the keyword plan ad group resource_name
+                these keyword plan keywords should be attributed to.
+
+        Raises:
+            GoogleAdsException: If an error is returned from the API.
+        """
+        data = {
+            "Keywords":list(),
+            "keyword_id":list(),
+            "keyword_type":list(),
+        }
+        keyword_plan_ad_group_keyword_service = client.get_service(
+            "KeywordPlanAdGroupKeywordService"
+        )
+        operation = client.get_type("KeywordPlanAdGroupKeywordOperation")
+        operations = []
+
+        for keyword in keywords:
+            operation = client.get_type("KeywordPlanAdGroupKeywordOperation")
+            keyword_plan_ad_group_keyword1 = operation.create
+            keyword_plan_ad_group_keyword1.text = keyword
+            keyword_plan_ad_group_keyword1.cpc_bid_micros = 1000000
+            keyword_plan_ad_group_keyword1.match_type = (
+                client.enums.KeywordMatchTypeEnum.EXACT
+            )
+            keyword_plan_ad_group_keyword1.keyword_plan_ad_group = plan_ad_group
+            data['Keywords'].append(keyword)
+            data['keyword_type'].append('EXACT')
+
+            operations.append(operation)
+
+            operation = client.get_type("KeywordPlanAdGroupKeywordOperation")
+            keyword_plan_ad_group_keyword2 = operation.create
+            keyword_plan_ad_group_keyword2.text = keyword
+            keyword_plan_ad_group_keyword2.cpc_bid_micros = 1000000
+            keyword_plan_ad_group_keyword2.match_type = (
+                client.enums.KeywordMatchTypeEnum.PHRASE
+            )
+            keyword_plan_ad_group_keyword2.keyword_plan_ad_group = plan_ad_group
+            operations.append(operation)
+
+            data['Keywords'].append(keyword)
+            data['keyword_type'].append('PHRASE')
+
+        response = keyword_plan_ad_group_keyword_service.mutate_keyword_plan_ad_group_keywords(
+            customer_id=customer_id, operations=operations
+        )
+
+        for result in response.results:
+            print(
+                "Created keyword plan ad group keyword with resource name: "
+                f"{result.resource_name}"
+            )
+        keywords_ids = [result.resource_name.split('/')[-1] for result in response.results]
+        data['keyword_id'] = keywords_ids
+        return data
+    
+    def _generate_forecast_metrics(client, customer_id, keyword_plan):
+        data = {
+            "metrics_impression":list(),
+            "metrics_cpc":list(),
+            "metrics_clicks":list(),
+            "metrics_cost":list(),
+            "metrics_ctr":list(),
+            "keyword_id":list(),
+        }
+        keyword_plan_service = client.get_service("KeywordPlanService")
+
+        response = keyword_plan_service.generate_forecast_metrics(
+            keyword_plan=keyword_plan
+        )
+
+        for forecast in response.keyword_forecasts:
+            metrics = forecast.keyword_forecast
+            data['metrics_clicks'].append(metrics.clicks)
+            data['metrics_impression'].append(metrics.impressions)
+            data['metrics_cpc'].append(metrics.average_cpc)
+            data['metrics_cost'].append(metrics.cost_micros)
+            data['metrics_ctr'].append(metrics.ctr)
+            data['keyword_id'].append(forecast.keyword_plan_ad_group_keyword.split('/')[-1])
+        
+        return data
+
+    keyword_plan = _create_keyword_plan(client, customer_id)
+    keyword_plan_campaign = _create_keyword_plan_campaign(
+        client, customer_id, keyword_plan
+    )
+    keyword_plan_ad_group = _create_keyword_plan_ad_group(
+        client, customer_id, keyword_plan_campaign
+    )
+    keyword_data = _create_keyword_plan_ad_group_keywords(
+        client, customer_id, keyword_plan_ad_group,keywords
+    )
+    data = _generate_forecast_metrics(client,customer_id,keyword_plan)
+    
+    keyword_data = pd.DataFrame(keyword_data)
+    data = pd.DataFrame(data)
+
+    data = data.merge(keyword_data,how='left',on='keyword_id')
+    return data
+
+
+
 def make_request(keyword, email, api_key):
     client = RestClient(email, api_key)
     post_data = dict()
@@ -630,7 +870,7 @@ def make_request(keyword, email, api_key):
               (response["status_code"], response["status_message"]))
 
 
-def make_live_request(keyword, email, api_key,priority = 2, depth = 2, max_crawl_pages = 2):
+def make_live_request(keyword, email, api_key, priority=2, depth=2, max_crawl_pages=2):
     client = RestClient(email, api_key)
     post_data = dict()
     post_data[len(post_data)] = dict(
@@ -654,9 +894,9 @@ def make_live_request(keyword, email, api_key,priority = 2, depth = 2, max_crawl
 
 def get_results(results, tasks, task_params):
     df = {
-        'organic_url' : list(),
-        'organic_description' : list(),
-        'organic_extended_people_also_search' : list(),
+        'organic_url': list(),
+        'organic_description': list(),
+        'organic_extended_people_also_search': list(),
     }
     for i, data in enumerate(results['tasks'][0]['result'][0]['items']):
         task = data['type']
@@ -668,7 +908,7 @@ def get_results(results, tasks, task_params):
 
         if task == 'people_also_ask' and task in tasks:
             all_items = {f'{task}_{param}': list()
-                                                 for param in task_params[task]}
+                         for param in task_params[task]}
             for item in data['items']:
                 all_items[f'{task}_title'].append(item[f'title'])
                 for key in item['expanded_element'][0].keys():
@@ -685,7 +925,7 @@ def get_results(results, tasks, task_params):
         if task == 'top_stories' and task in tasks:
             # [domain, title, date, url]
             all_items = {f'{task}_{param}': list()
-                                                 for param in task_params[task]}
+                         for param in task_params[task]}
             all_items[f'{task}_title'].append(data['title'])
             for item in data['items']:
                 for key in item.keys():
@@ -697,7 +937,7 @@ def get_results(results, tasks, task_params):
 
         if task == 'knowledge_graph' and task in tasks:
             all_items = {f'{task}_{param}': list()
-                                                 for param in task_params[task]}
+                         for param in task_params[task]}
             #[title, url, text]
             all_items.update({f'{task}_main_title': list(),
                              f'{task}_description': list()})
@@ -706,13 +946,14 @@ def get_results(results, tasks, task_params):
             for item in data['items']:
                 try:
                     all_items[f'{task}_text'].append(item[f'text'])
-                        
+
                 except:
                     all_items[f'{task}_text'].append(None)
                     for key in item.keys():
                         if key in task_params[f'{task}'] and key != 'text':
                             try:
-                                all_items[f'{task}_{key}'].append(item['links'][0][f'{key}'])
+                                all_items[f'{task}_{key}'].append(
+                                    item['links'][0][f'{key}'])
                             except:
                                 all_items[f'{task}_{key}'].append(None)
             all_items = {k: str(v) for k, v in all_items.items()}
